import unittest

from requests_mock import Mocker

from coinbase.rest import RESTClient

from ..constants import TEST_API_KEY, TEST_API_SECRET


class DataApiTest(unittest.TestCase):
    def test_get_api_key_permissions(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {
            "can_view": True,
            "can_trade": False,
            "can_withdraw": False,
            "portfolio_uuid": "portfolio1",
        }

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/key_permissions",
                json=expected_response,
            )
            key_permissions = client.get_api_key_permissions()

            self.assertEqual(key_permissions, expected_response)



import unittest

from requests_mock import Mocker

from coinbase.rest import RESTClient

from ..constants import TEST_API_KEY, TEST_API_SECRET


class ConvertTest(unittest.TestCase):
    def test_create_convert_quote(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"quote_id": "1234"}

        with Mocker() as m:
            m.request(
                "POST",
                "https://api.coinbase.com/api/v3/brokerage/convert/quote",
                json=expected_response,
            )
            quote = client.create_convert_quote(
                "from_account",
                "to_account",
                "100",
                user_incentive_id="1234",
                code_val="test_val",
            )

            captured_request = m.request_history[0]
            captured_json = captured_request.json()

            self.assertEqual(captured_request.query, "")
            self.assertEqual(
                captured_json,
                {
                    "from_account": "from_account",
                    "to_account": "to_account",
                    "amount": "100",
                    "trade_incentive_metadata": {
                        "user_incentive_id": "1234",
                        "code_val": "test_val",
                    },
                },
            )
            self.assertEqual(quote, expected_response)

    def test_get_convert_trade(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"trade_id": "1234"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/convert/trade/1234",
                json=expected_response,
            )
            trade = client.get_convert_trade("1234", "from_account", "to_account")

            captured_request = m.request_history[0]

            self.assertEqual(
                captured_request.query,
                "from_account=from_account&to_account=to_account",
            )
            self.assertEqual(trade, expected_response)

    def test_commit_convert_trade(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"trade_id": "1234"}

        with Mocker() as m:
            m.request(
                "POST",
                "https://api.coinbase.com/api/v3/brokerage/convert/trade/1234",
                json=expected_response,
            )
            trade = client.commit_convert_trade("1234", "from_account", "to_account")

            captured_request = m.request_history[0]
            captured_json = captured_request.json()

            self.assertEqual(captured_request.query, "")
            self.assertEqual(
                captured_json,
                {"from_account": "from_account", "to_account": "to_account"},
            )
            self.assertEqual(trade, expected_response)



from typing import Any, Dict, Optional

from coinbase.constants import API_PREFIX


def get_api_key_permissions(
    self,
    **kwargs,
) -> Dict[str, Any]:
    """
    **Get Api Key Permissions**
    _____________________________

    [GET] https://api.coinbase.com/api/v3/brokerage/key_permissions

    __________

    **Description:**

    Get information about your CDP API key permissions

    __________

    **Read more on the official documentation:** `Create Convert Quote <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getapikeypermissions>`_
    """
    endpoint = f"{API_PREFIX}/key_permissions"

    return self.get(endpoint, **kwargs)



import unittest

from requests_mock import Mocker

from coinbase.rest import RESTClient

from ..constants import TEST_API_KEY, TEST_API_SECRET


class FuturesTest(unittest.TestCase):
    def test_get_futures_balance_summary(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/cfm/balance_summary",
                json=expected_response,
            )
            balance_summary = client.get_futures_balance_summary()

            captured_request = m.request_history[0]

            self.assertEqual(captured_request.query, "")
            self.assertEqual(balance_summary, expected_response)

    def test_list_futures_positions(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/cfm/positions",
                json=expected_response,
            )
            positions = client.list_futures_positions()

            captured_request = m.request_history[0]

            self.assertEqual(captured_request.query, "")
            self.assertEqual(positions, expected_response)

    def test_get_futures_position(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/cfm/positions/PRODUCT_ID_1",
                json=expected_response,
            )
            position = client.get_futures_position("PRODUCT_ID_1")

            captured_request = m.request_history[0]

            self.assertEqual(captured_request.query, "")
            self.assertEqual(position, expected_response)

    def test_schedule_futures_sweep(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "POST",
                "https://api.coinbase.com/api/v3/brokerage/cfm/sweeps/schedule",
                json=expected_response,
            )
            response = client.schedule_futures_sweep("5")

            captured_request = m.request_history[0]
            captured_json = captured_request.json()

            self.assertEqual(captured_request.query, "")
            self.assertEqual(captured_json, {"usd_amount": "5"})
            self.assertEqual(response, expected_response)

    def test_list_futures_sweeps(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/cfm/sweeps",
                json=expected_response,
            )
            sweeps = client.list_futures_sweeps()

            captured_request = m.request_history[0]

            self.assertEqual(captured_request.query, "")
            self.assertEqual(sweeps, expected_response)

    def test_cancel_pending_futures_sweep(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "DELETE",
                "https://api.coinbase.com/api/v3/brokerage/cfm/sweeps",
                json=expected_response,
            )
            delete = client.cancel_pending_futures_sweep()

            captured_request = m.request_history[0]

            self.assertEqual(captured_request.query, "")
            self.assertEqual(delete, expected_response)

    def test_get_intraday_margin_setting(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/cfm/intraday/margin_setting",
                json=expected_response,
            )
            intraday_margin_setting = client.get_intraday_margin_setting()

            captured_request = m.request_history[0]

            self.assertEqual(captured_request.query, "")
            self.assertEqual(intraday_margin_setting, expected_response)

    def test_get_current_margin_window(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/cfm/intraday/current_margin_window",
                json=expected_response,
            )
            margin_window = client.get_current_margin_window("MARGIN_PROFILE_TYPE_1")

            captured_request = m.request_history[0]

            self.assertEqual(
                captured_request.query, "margin_profile_type=margin_profile_type_1"
            )
            self.assertEqual(margin_window, expected_response)

    def test_set_intraday_margin_setting(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "POST",
                "https://api.coinbase.com/api/v3/brokerage/cfm/intraday/margin_setting",
                json=expected_response,
            )
            setting = client.set_intraday_margin_setting("setting_1")

            captured_request = m.request_history[0]
            captured_json = captured_request.json()

            self.assertEqual(captured_request.query, "")

            self.assertEqual(
                captured_json,
                {
                    "setting": "setting_1",
                },
            )

            self.assertEqual(setting, expected_response)



from typing import Any, Dict, List, Optional

from coinbase.constants import API_PREFIX


def get_products(
    self,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    product_type: Optional[str] = None,
    product_ids: Optional[List[str]] = None,
    contract_expiry_type: Optional[str] = None,
    expiring_contract_status: Optional[str] = None,
    get_tradability_status: Optional[bool] = False,
    get_all_products: Optional[bool] = False,
    **kwargs,
) -> Dict[str, Any]:
    """
    **List Products**
    _________________

    [GET] https://api.coinbase.com/api/v3/brokerage/products

    __________

    **Description:**

    Get a list of the available currency pairs for trading.

    __________

    **Read more on the official documentation:** `List Products
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getproducts>`_
    """
    endpoint = f"{API_PREFIX}/products"

    params = {
        "limit": limit,
        "offset": offset,
        "product_type": product_type,
        "product_ids": product_ids,
        "contract_expiry_type": contract_expiry_type,
        "expiring_contract_status": expiring_contract_status,
        "get_tradability_status": get_tradability_status,
        "get_all_products": get_all_products,
    }

    return self.get(endpoint, params=params, **kwargs)


def get_product(
    self, product_id: str, get_tradability_status: Optional[bool] = False, **kwargs
) -> Dict[str, Any]:
    """
    **Get Product**
    _______________

    [GET] https://api.coinbase.com/api/v3/brokerage/products/{product_id}

    __________

    **Description:**

    Get information on a single product by product ID.

    __________

    **Read more on the official documentation:** `Get Product
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getproduct>`_
    """
    endpoint = f"{API_PREFIX}/products/{product_id}"

    params = {
        "get_tradability_status": get_tradability_status,
    }

    return self.get(endpoint, params=params, **kwargs)


def get_product_book(
    self,
    product_id: str,
    limit: Optional[int] = None,
    aggregation_price_increment: Optional[str] = None,
    **kwargs,
) -> Dict[str, Any]:
    """
    **Get Product Book**
    ____________________

    [GET] https://api.coinbase.com/api/v3/brokerage/product_book

    __________

    **Description:**

    Get a list of bids/asks for a single product. The amount of detail shown can be customized with the limit parameter.

    __________

    **Read more on the official documentation:** `Get Product Book
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getproductbook>`_
    """
    endpoint = f"{API_PREFIX}/product_book"

    params = {
        "product_id": product_id,
        "limit": limit,
        "aggregation_price_increment": aggregation_price_increment,
    }

    return self.get(endpoint, params=params, **kwargs)


def get_best_bid_ask(
    self, product_ids: Optional[List[str]] = None, **kwargs
) -> Dict[str, Any]:
    """
    **Get Best Bid/Ask**
    ____________________

    [GET] https://api.coinbase.com/api/v3/brokerage/best_bid_ask

    __________

    **Description:**

    Get the best bid/ask for all products. A subset of all products can be returned instead by using the product_ids input.

    __________

    **Read more on the official documentation:** `Get Best Bid/Ask
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getproductbook>`_
    """
    endpoint = f"{API_PREFIX}/best_bid_ask"

    params = {
        "product_ids": product_ids,
    }

    return self.get(endpoint, params=params, **kwargs)



from coinbase.__version__ import __version__

API_ENV_KEY = "COINBASE_API_KEY"
API_SECRET_ENV_KEY = "COINBASE_API_SECRET"
USER_AGENT = f"coinbase-advanced-py/{__version__}"

# REST Constants
BASE_URL = "api.coinbase.com"
API_PREFIX = "/api/v3/brokerage"

# Websocket Constants
WS_BASE_URL = "wss://advanced-trade-ws.coinbase.com"
WS_USER_BASE_URL = "wss://advanced-trade-ws-user.coinbase.com"

WS_RETRY_MAX = 5
WS_RETRY_BASE = 5
WS_RETRY_FACTOR = 1.5

# Message Types
SUBSCRIBE_MESSAGE_TYPE = "subscribe"
UNSUBSCRIBE_MESSAGE_TYPE = "unsubscribe"

# Channels
HEARTBEATS = "heartbeats"
CANDLES = "candles"
MARKET_TRADES = "market_trades"
STATUS = "status"
TICKER = "ticker"
TICKER_BATCH = "ticker_batch"
LEVEL2 = "level2"
USER = "user"
FUTURES_BALANCE_SUMMARY = "futures_balance_summary"

WS_AUTH_CHANNELS = {USER, FUTURES_BALANCE_SUMMARY}

X_RATELIMIT_LIMIT = "x-ratelimit-limit"
X_RATELIMIT_REMAINING = "x-ratelimit-remaining"
X_RATELIMIT_RESET = "x-ratelimit-reset"
RATE_LIMIT_HEADERS = {X_RATELIMIT_LIMIT, X_RATELIMIT_REMAINING, X_RATELIMIT_RESET}



from typing import Any, Dict, Optional

from coinbase.constants import API_PREFIX


def get_candles(
    self,
    product_id: str,
    start: str,
    end: str,
    granularity: str,
    limit: Optional[int] = None,
    **kwargs,
) -> Dict[str, Any]:
    """
    **Get Product Candles**
    __________

    [GET] https://api.coinbase.com/api/v3/brokerage/products/{product_id}/candles

    __________

    **Description:**

    Get rates for a single product by product ID, grouped in buckets.

    __________

    **Read more on the official documentation:** `Get Product Candles
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getcandles>`_
    """
    endpoint = f"{API_PREFIX}/products/{product_id}/candles"

    params = {"start": start, "end": end, "granularity": granularity, "limit": limit}

    return self.get(endpoint, params=params, **kwargs)


def get_market_trades(
    self,
    product_id: str,
    limit: int,
    start: Optional[str] = None,
    end: Optional[str] = None,
    **kwargs,
) -> Dict[str, Any]:
    """
    **Get Market Trades**
    _____________________

    [GET] https://api.coinbase.com/api/v3/brokerage/products/{product_id}/ticker

    __________

    **Description:**

    Get snapshot information, by product ID, about the last trades (ticks), best bid/ask, and 24h volume.

    __________

    **Read more on the official documentation:** `Get Market Trades
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getmarkettrades>`_
    """
    endpoint = f"{API_PREFIX}/products/{product_id}/ticker"

    params = {"limit": limit, "start": start, "end": end}

    return self.get(endpoint, params=params, **kwargs)



import unittest

from requests_mock import Mocker

from coinbase.rest import RESTClient

from ..constants import TEST_API_KEY, TEST_API_SECRET


class ProductsTest(unittest.TestCase):
    def test_get_products(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/products",
                json=expected_response,
            )
            products = client.get_products(limit=2, product_type="SPOT")

            captured_request = m.request_history[0]

            self.assertEqual(
                captured_request.query,
                "limit=2&product_type=spot&get_tradability_status=false&get_all_products=false",
            )
            self.assertEqual(products, expected_response)

    def test_get_product(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"product_id": "product_1"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/products/product_1",
                json=expected_response,
            )
            product = client.get_product("product_1")

            captured_request = m.request_history[0]

            self.assertEqual(captured_request.query, "get_tradability_status=false")
            self.assertEqual(product, expected_response)

    def test_get_product_book(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/product_book",
                json=expected_response,
            )
            book = client.get_product_book("product_1", 10)

            captured_request = m.request_history[0]

            self.assertEqual(captured_request.query, "product_id=product_1&limit=10")
            self.assertEqual(book, expected_response)

    def test_get_best_bid_ask(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/best_bid_ask",
                json=expected_response,
            )
            bid_ask = client.get_best_bid_ask(["product_1", "product_2"])

            captured_request = m.request_history[0]

            self.assertEqual(
                captured_request.query, "product_ids=product_1&product_ids=product_2"
            )
            self.assertEqual(bid_ask, expected_response)



from typing import Any, Dict, Optional

from coinbase.constants import API_PREFIX


def get_futures_balance_summary(self, **kwargs) -> Dict[str, Any]:
    """
    **Get Futures Balance Summary**
    _______________________________

    [GET] https://api.coinbase.com/api/v3/brokerage/cfm/balance_summary

    __________

    **Description:**

    Get information on your balances related to `Coinbase Financial Markets <https://www.coinbase.com/fcm>`_ (CFM) futures trading.

    __________

    **Read more on the official documentation:** `Get Futures Balance Summary
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getfcmbalancesummary>`_
    """
    endpoint = f"{API_PREFIX}/cfm/balance_summary"

    return self.get(endpoint, **kwargs)


def list_futures_positions(self, **kwargs) -> Dict[str, Any]:
    """
    **List Futures Positions**
    __________________________

    [GET] https://api.coinbase.com/api/v3/brokerage/cfm/positions

    __________

    **Description:**

    Get a list of all open positions in CFM futures products.

    __________

    **Read more on the official documentation:** `List Futures Positions
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getfcmpositions>`_
    """
    endpoint = f"{API_PREFIX}/cfm/positions"

    return self.get(endpoint, **kwargs)


def get_futures_position(self, product_id: str, **kwargs) -> Dict[str, Any]:
    """
    **Get Futures Position**
    _________________________

    [GET] https://api.coinbase.com/api/v3/brokerage/cfm/positions/{product_id}

    __________

    **Description:**

    Get the position of a specific CFM futures product.

    __________

    **Read more on the official documentation:** `Get Futures Position
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getfcmposition>`_
    """
    endpoint = f"{API_PREFIX}/cfm/positions/{product_id}"

    return self.get(endpoint, **kwargs)


def schedule_futures_sweep(self, usd_amount: str, **kwargs) -> Dict[str, Any]:
    """
    **Schedule Futures Sweep**
    __________________________

    [POST] https://api.coinbase.com/api/v3/brokerage/cfm/sweeps/schedule

    __________

    **Description:**

    Schedule a sweep of funds from your CFTC-regulated futures account to your Coinbase Inc. USD Spot wallet.

    __________

    **Read more on the official documentation:** `Schedule Futures Sweep
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_schedulefcmsweep>`_
    """
    endpoint = f"{API_PREFIX}/cfm/sweeps/schedule"

    data = {"usd_amount": usd_amount}

    return self.post(endpoint, data=data, **kwargs)


def list_futures_sweeps(self, **kwargs) -> Dict[str, Any]:
    """
    **List Futures Sweeps**
    _______________________

    [GET] https://api.coinbase.com/api/v3/brokerage/cfm/sweeps

    __________

    **Description:**

    Get information on your pending and/or processing requests to sweep funds from your CFTC-regulated futures account to your Coinbase Inc. USD Spot wallet.

    __________

    **Read more on the official documentation:** `List Futures Sweeps
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getfcmsweeps>`_
    """
    endpoint = f"{API_PREFIX}/cfm/sweeps"

    return self.get(endpoint, **kwargs)


def cancel_pending_futures_sweep(self, **kwargs) -> Dict[str, Any]:
    """
    **Cancel Pending Futures Sweep**
    ________________________________

    [DELETE] https://api.coinbase.com/api/v3/brokerage/cfm/sweeps

    __________

    **Description:**

    Cancel your pending sweep of funds from your CFTC-regulated futures account to your Coinbase Inc. USD Spot wallet.

    __________

    **Read more on the official documentation:** `Cancel Pending Futures Sweep
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_cancelfcmsweep>`_
    """
    endpoint = f"{API_PREFIX}/cfm/sweeps"

    return self.delete(endpoint, **kwargs)


def get_intraday_margin_setting(self, **kwargs) -> Dict[str, Any]:
    """
    **Get Intraday Margin Setting**
    _______________________________

    [GET] https://api.coinbase.com/api/v3/brokerage/cfm/intraday/margin_setting

    __________

    **Description:**

    Get the status of whether your account is opted in to receive increased leverage on futures trades on weekdays from 8am-4pm ET.

    __________

    **Read more on the official documentation:** `Get Intraday Margin Setting
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getintradaymarginsetting>`_
    """
    endpoint = f"{API_PREFIX}/cfm/intraday/margin_setting"

    return self.get(endpoint, **kwargs)


def get_current_margin_window(
    self, margin_profile_type: str, **kwargs
) -> Dict[str, Any]:
    """
    **Get Current Margin Window**
    ________________________________

    [GET] https://api.coinbase.com/api/v3/brokerage/cfm/intraday/current_margin_window

    __________

    **Description:**

    Get the current margin window to determine whether intraday or overnight margin rates are in effect.

    __________

    **Read more on the official documentation:** `Get Current Margin Window
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getcurrentmarginwindow>`_
    """

    endpoint = f"{API_PREFIX}/cfm/intraday/current_margin_window"

    params = {"margin_profile_type": margin_profile_type}

    return self.get(endpoint, params=params, **kwargs)


def set_intraday_margin_setting(self, setting: str, **kwargs) -> Dict[str, Any]:
    """
    **Set Intraday Margin Setting**
    ________________________________

    [POST] https://api.coinbase.com/api/v3/brokerage/cfm/intraday/margin_setting

    __________

    **Description:**

    Opt in to receive increased leverage on futures trades on weekdays from 8am-4pm ET.

    __________

    **Read more on the official documentation:** `Set Intraday Margin Setting
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_setintradaymarginsetting>`_
    """

    endpoint = f"{API_PREFIX}/cfm/intraday/margin_setting"

    data = {"setting": setting}

    return self.post(endpoint, data=data, **kwargs)



from typing import Any, Dict, Optional

from coinbase.constants import API_PREFIX


def allocate_portfolio(
    self, portfolio_uuid: str, symbol: str, amount: str, currency: str, **kwargs
) -> Dict[str, Any]:
    """
    **Allocate Portfolio**
    ________________

    [POST] https://api.coinbase.com/api/v3/brokerage/intx/allocate

    __________

    **Description:**

    Allocate more funds to an isolated position in your Perpetuals portfolio.

    __________

    **Read more on the official documentation:** `Allocate Portfolio
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_allocateportfolio>`_
    """

    endpoint = f"{API_PREFIX}/intx/allocate"

    data = {
        "portfolio_uuid": portfolio_uuid,
        "symbol": symbol,
        "amount": amount,
        "currency": currency,
    }

    return self.post(endpoint, data=data, **kwargs)


def get_perps_portfolio_summary(self, portfolio_uuid: str, **kwargs) -> Dict[str, Any]:
    """
    **Get Perpetuals Portfolio Summary**
    ________________

    [GET] https://api.coinbase.com/api/v3/brokerage/intx/portfolio/{portfolio_uuid}

    __________

    **Description:**

    Get a summary of your Perpetuals portfolio.

    __________

    **Read more on the official documentation:** `Get Perpetuals Portfolio Summary
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getintxportfoliosummary>`_
    """
    endpoint = f"{API_PREFIX}/intx/portfolio/{portfolio_uuid}"

    return self.get(endpoint, **kwargs)


def list_perps_positions(self, portfolio_uuid: str, **kwargs) -> Dict[str, Any]:
    """
    **List Perpetuals Positions**
    ________________

    [GET] https://api.coinbase.com/api/v3/brokerage/intx/positions/{portfolio_uuid}

    __________

    **Description:**

    Get a list of open positions in your Perpetuals portfolio.

    __________

    **Read more on the official documentation:** `List Perpetuals Positions
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getintxpositions>`_
    """
    endpoint = f"{API_PREFIX}/intx/positions/{portfolio_uuid}"

    return self.get(endpoint, **kwargs)


def get_perps_position(
    self, portfolio_uuid: str, symbol: str, **kwargs
) -> Dict[str, Any]:
    """
    **Get Perpetuals Position**
    ________________

    [GET] https://api.coinbase.com/api/v3/brokerage/intx/positions/{portfolio_uuid}/{symbol}

    __________

    **Description:**

    Get a specific open position in your Perpetuals portfolio

    __________

    **Read more on the official documentation:** `Get Perpetuals Positions
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getintxposition>`_
    """
    endpoint = f"{API_PREFIX}/intx/positions/{portfolio_uuid}/{symbol}"

    return self.get(endpoint, **kwargs)


def get_perps_portfolio_balances(self, portfolio_uuid: str, **kwargs) -> Dict[str, Any]:
    """
    **Get Portfolio Balances**
    ________________

    [GET] https://api.coinbase.com/api/v3/brokerage/intx/balances/{portfolio_uuid}

    __________

    **Description:**

    Get a list of asset balances on Intx for a given Portfolio

    __________

    **Read more on the official documentation:** `Get Portfolio Balances
    <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getintxbalances>`_
    """
    endpoint = f"{API_PREFIX}/intx/balances/{portfolio_uuid}"

    return self.get(endpoint, **kwargs)


def opt_in_or_out_multi_asset_collateral(
    self, portfolio_uuid: str, multi_asset_collateral_enabled: bool, **kwargs
) -> Dict[str, Any]:
    """
    **Opt In or Out of Multi Asset Collateral**
    ________________

    [POST] https://api.coinbase.com/api/v3/brokerage/intx/multi_asset_collateral

    __________

    **Description:**

    Enable or Disable Multi Asset Collateral for a given Portfolio.

    __________

    **Read more on the official documentation:** `Opt In or Out of Multi Asset Collateral
    <https://docs.cloud.coinbase.com/advanced-trade-api/reference/retailbrokerageapi_intxmultiassetcollateral>`_
    """

    endpoint = f"{API_PREFIX}/intx/multi_asset_collateral"

    data = {
        "portfolio_uuid": portfolio_uuid,
        "multi_asset_collateral_enabled": multi_asset_collateral_enabled,
    }

    return self.post(endpoint, data=data, **kwargs)



import unittest

from requests_mock import Mocker

from coinbase.rest import RESTClient

from ..constants import TEST_API_KEY, TEST_API_SECRET


class FeesTest(unittest.TestCase):
    def test_get_transaction_summary(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/transaction_summary",
                json=expected_response,
            )
            summary = client.get_transaction_summary(
                "product_type", "contract_expiry_type", "product_venue"
            )

            captured_request = m.request_history[0]

            self.assertEqual(
                captured_request.query,
                "product_type=product_type&contract_expiry_type=contract_expiry_type&product_venue=product_venue",
            )
            self.assertEqual(summary, expected_response)



import secrets
import time

import jwt
from cryptography.hazmat.primitives import serialization

from coinbase.constants import BASE_URL


def build_jwt(key_var, secret_var, uri=None) -> str:
    """
    :meta private:
    """
    try:
        private_key_bytes = secret_var.encode("utf-8")
        private_key = serialization.load_pem_private_key(
            private_key_bytes, password=None
        )
    except ValueError as e:
        # This handles errors like incorrect key format
        raise Exception(
            f"{e}\n"
            "Are you sure you generated your key at https://cloud.coinbase.com/access/api ?"
        )

    jwt_data = {
        "sub": key_var,
        "iss": "cdp",
        "nbf": int(time.time()),
        "exp": int(time.time()) + 120,
    }

    if uri:
        jwt_data["uri"] = uri

    jwt_token = jwt.encode(
        jwt_data,
        private_key,
        algorithm="ES256",
        headers={"kid": key_var, "nonce": secrets.token_hex()},
    )

    return jwt_token


def build_rest_jwt(uri, key_var, secret_var) -> str:
    """
    **Build REST JWT**
    __________

    **Description:**

    Builds and returns a JWT token for connecting to the REST API.

    __________

    Parameters:

    - **uri (str)** - Formatted URI for the endpoint (e.g. "GET api.coinbase.com/api/v3/brokerage/accounts") Can be generated using ``format_jwt_uri``
    - **key_var (str)** - The API key
    - **secret_var (str)** - The API key secret
    """
    return build_jwt(key_var, secret_var, uri=uri)


def build_ws_jwt(key_var, secret_var) -> str:
    """
    **Build WebSocket JWT**
    __________

    **Description:**

    Builds and returns a JWT token for connecting to the WebSocket API.

    __________

    Parameters:

    - **key_var (str)** - The API key
    - **secret_var (str)** - The API key secret
    """
    return build_jwt(key_var, secret_var)


def format_jwt_uri(method, path) -> str:
    """
    **Format JWT URI**
    __________

    **Description:**

    Formats method and path into valid URI for JWT generation.

    __________

    Parameters:

    - **method (str)** - The REST request method. E.g. GET, POST, PUT, DELETE
    - **path (str)** - The path of the endpoint. E.g. "/api/v3/brokerage/accounts"

    """
    return f"{method} {BASE_URL}{path}"



from .rest_base import RESTBase


class RESTClient(RESTBase):
    """
    **RESTClient**
    _____________________________

    Initialize using RESTClient

    __________

    **Parameters**:

    - **api_key | Optional (str)** - The API key
    - **api_secret | Optional (str)** - The API key secret
    - **key_file | Optional (IO | str)** - Path to API key file or file-like object
    - **base_url | (str)** - The base URL for REST requests. Default set to "https://api.coinbase.com"
    - **timeout | Optional (int)** - Set timeout in seconds for REST requests
    - **verbose | Optional (bool)** - Enables debug logging. Default set to False


    """

    from .accounts import get_account, get_accounts
    from .convert import commit_convert_trade, create_convert_quote, get_convert_trade
    from .data_api import get_api_key_permissions
    from .fees import get_transaction_summary
    from .futures import (
        cancel_pending_futures_sweep,
        get_current_margin_window,
        get_futures_balance_summary,
        get_futures_position,
        get_intraday_margin_setting,
        list_futures_positions,
        list_futures_sweeps,
        schedule_futures_sweep,
        set_intraday_margin_setting,
    )
    from .market_data import get_candles, get_market_trades
    from .orders import (
        cancel_orders,
        close_position,
        create_order,
        edit_order,
        get_fills,
        get_order,
        limit_order_fok,
        limit_order_fok_buy,
        limit_order_fok_sell,
        limit_order_gtc,
        limit_order_gtc_buy,
        limit_order_gtc_sell,
        limit_order_gtd,
        limit_order_gtd_buy,
        limit_order_gtd_sell,
        limit_order_ioc,
        limit_order_ioc_buy,
        limit_order_ioc_sell,
        list_orders,
        market_order,
        market_order_buy,
        market_order_sell,
        preview_edit_order,
        preview_limit_order_fok,
        preview_limit_order_fok_buy,
        preview_limit_order_fok_sell,
        preview_limit_order_gtc,
        preview_limit_order_gtc_buy,
        preview_limit_order_gtc_sell,
        preview_limit_order_gtd,
        preview_limit_order_gtd_buy,
        preview_limit_order_gtd_sell,
        preview_limit_order_ioc,
        preview_limit_order_ioc_buy,
        preview_limit_order_ioc_sell,
        preview_market_order,
        preview_market_order_buy,
        preview_market_order_sell,
        preview_order,
        preview_stop_limit_order_gtc,
        preview_stop_limit_order_gtc_buy,
        preview_stop_limit_order_gtc_sell,
        preview_stop_limit_order_gtd,
        preview_stop_limit_order_gtd_buy,
        preview_stop_limit_order_gtd_sell,
        preview_trigger_bracket_order_gtc,
        preview_trigger_bracket_order_gtc_buy,
        preview_trigger_bracket_order_gtc_sell,
        preview_trigger_bracket_order_gtd,
        preview_trigger_bracket_order_gtd_buy,
        preview_trigger_bracket_order_gtd_sell,
        stop_limit_order_gtc,
        stop_limit_order_gtc_buy,
        stop_limit_order_gtc_sell,
        stop_limit_order_gtd,
        stop_limit_order_gtd_buy,
        stop_limit_order_gtd_sell,
        trigger_bracket_order_gtc,
        trigger_bracket_order_gtc_buy,
        trigger_bracket_order_gtc_sell,
        trigger_bracket_order_gtd,
        trigger_bracket_order_gtd_buy,
        trigger_bracket_order_gtd_sell,
    )
    from .payments import get_payment_method, list_payment_methods
    from .perpetuals import (
        allocate_portfolio,
        get_perps_portfolio_balances,
        get_perps_portfolio_summary,
        get_perps_position,
        list_perps_positions,
        opt_in_or_out_multi_asset_collateral,
    )
    from .portfolios import (
        create_portfolio,
        delete_portfolio,
        edit_portfolio,
        get_portfolio_breakdown,
        get_portfolios,
        move_portfolio_funds,
    )
    from .products import get_best_bid_ask, get_product, get_product_book, get_products
    from .public import (
        get_public_candles,
        get_public_market_trades,
        get_public_product,
        get_public_product_book,
        get_public_products,
        get_unix_time,
    )



import unittest
from io import StringIO

from coinbase.api_base import APIBase

from .constants import TEST_API_KEY, TEST_API_SECRET


class RestBaseTest(unittest.TestCase):
    def test_no_api_key(self):
        with self.assertRaises(Exception):
            APIBase("test_key", None)

    def test_key_api_key_vars(self):
        try:
            APIBase(api_key=TEST_API_KEY, api_secret=TEST_API_SECRET)
        except Exception as e:
            self.fail(f"An unexpected exception occurred: {e}")

    def test_key_file_string(self):
        try:
            APIBase(key_file="tests/test_api_key.json")
        except Exception as e:
            self.fail(f"An unexpected exception occurred: {e}")

    def test_key_file_object(self):
        try:
            key_file_object = StringIO(
                '{"name": "test-api-key-name","privateKey": "test-api-key-private-key"}'
            )
            APIBase(key_file=key_file_object)
        except Exception as e:
            self.fail(f"An unexpected exception occurred: {e}")

    def test_key_file_no_key(self):
        with self.assertRaises(Exception):
            key_file_object = StringIO('{"field_1": "value_1","field_2": "value_2"}')
            APIBase(key_file=key_file_object)

    def test_key_file_multiple_key_inputs(self):
        with self.assertRaises(Exception):
            key_file_object = StringIO('{"field_1": "value_1","field_2": "value_2"}')
            APIBase(
                api_key=TEST_API_KEY,
                api_secret=TEST_API_SECRET,
                key_file=key_file_object,
            )

    def test_key_file_invalid_json(self):
        with self.assertRaises(Exception):
            key_file_object = StringIO(
                '"name": "test-api-key-name","privateKey": "test-api-key-private-key"'
            )
            APIBase(key_file=key_file_object)



import unittest

from requests_mock import Mocker

from coinbase.rest import RESTClient

from ..constants import TEST_API_KEY, TEST_API_SECRET


class MarketDataTest(unittest.TestCase):
    def test_get_candles(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/products/product_id_1/candles",
                json=expected_response,
            )
            candles = client.get_candles(
                "product_id_1", "1640995200", "1641081600", "FIVE_MINUTE", 2
            )

            captured_request = m.request_history[0]

            self.assertEqual(
                captured_request.query,
                "start=1640995200&end=1641081600&granularity=five_minute&limit=2",
            )
            self.assertEqual(candles, expected_response)

    def test_get_market_trades(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"key_1": "value_1", "key_2": "value_2"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/products/product_id/ticker",
                json=expected_response,
            )
            trades = client.get_market_trades(
                "product_id", 10, "1640995200", "1641081600"
            )

            captured_request = m.request_history[0]

            self.assertEqual(
                captured_request.query, "limit=10&start=1640995200&end=1641081600"
            )
            self.assertEqual(trades, expected_response)



import os

from setuptools import find_packages, setup

root = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(root, "requirements.txt"), "r") as fh:
    requirements = fh.readlines()

with open(os.path.join(root, "test_requirements.txt"), "r") as fh:
    test_requirements = fh.readlines()

with open(os.path.join(root, "lint_requirements.txt"), "r") as fh:
    lint_requirements = fh.readlines()

README = open(os.path.join(root, "README.md"), "r").read()

about = {}

with open(os.path.join(root, "coinbase", "__version__.py")) as f:
    exec(f.read(), about)

setup(
    name="coinbase-advanced-py",
    version=about["__version__"],
    license="Apache 2.0",
    description="Coinbase Advanced API Python SDK",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Coinbase",
    url="https://github.com/coinbase/coinbase-advanced-py",
    keywords=["Coinbase", "Advanced Trade", "API", "Advanced API"],
    packages=find_packages(exclude=("tests",)),
    install_requires=[req for req in requirements],
    extras_require={
        "test": [test_req for test_req in test_requirements],
        "lint": [lint_req for lint_req in lint_requirements],
    },
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Financial and Insurance Industry",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    python_requires=">=3.8",
)



import asyncio
import json
import time
import unittest
from unittest.mock import AsyncMock, patch

import websockets

from coinbase.constants import (
    CANDLES,
    FUTURES_BALANCE_SUMMARY,
    HEARTBEATS,
    LEVEL2,
    MARKET_TRADES,
    STATUS,
    TICKER,
    TICKER_BATCH,
    USER,
)
from coinbase.websocket import WSClient

from ..constants import TEST_API_KEY, TEST_API_SECRET

NO_PRODUCT_CHANNELS = {HEARTBEATS, FUTURES_BALANCE_SUMMARY}


class WSBaseTest(unittest.IsolatedAsyncioTestCase):
    async def asyncSetUp(self):
        self.on_message_mock = unittest.mock.Mock()

        # set up mock websocket messages
        connection_closed_exception = websockets.ConnectionClosedOK(
            1000, "Normal closure", False
        )
        self.mock_websocket = AsyncMock()
        self.mock_websocket.recv = AsyncMock(
            side_effect=[
                connection_closed_exception,
            ]
        )

        # initialize client
        self.ws = WSClient(
            TEST_API_KEY, TEST_API_SECRET, on_message=self.on_message_mock
        )

    @patch("websockets.connect", new_callable=AsyncMock)
    def generic_channel_test(
        self, channel_func, channel_func_unsub, channel_const, mock_connect
    ):
        # assert you can subscribe and unsubscribe to a channel
        mock_connect.return_value = self.mock_websocket

        # open
        self.ws.open()
        self.assertIsNotNone(self.ws.websocket)

        # subscribe
        product_ids = []
        if channel_const not in NO_PRODUCT_CHANNELS:
            product_ids = ["BTC-USD", "ETH-USD"]
            channel_func(product_ids=product_ids)
        else:
            channel_func()
        self.mock_websocket.send.assert_awaited_once()

        # assert subscribe message
        subscribe = json.loads(self.mock_websocket.send.call_args_list[0][0][0])
        self.assertEqual(subscribe["type"], "subscribe")
        self.assertEqual(subscribe["product_ids"], product_ids)
        self.assertEqual(subscribe["channel"], channel_const)

        # unsubscribe
        if channel_const not in NO_PRODUCT_CHANNELS:
            channel_func_unsub(product_ids=product_ids)
        else:
            channel_func_unsub()
        self.assertEqual(self.mock_websocket.send.await_count, 2)

        # assert unsubscribe message
        unsubscribe = json.loads(self.mock_websocket.send.call_args_list[1][0][0])
        self.assertEqual(unsubscribe["type"], "unsubscribe")
        self.assertEqual(unsubscribe["product_ids"], product_ids)
        self.assertEqual(unsubscribe["channel"], channel_const)

        # close
        self.ws.close()
        self.mock_websocket.close.assert_awaited_once()

    @patch("websockets.connect", new_callable=AsyncMock)
    async def generic_channel_test_async(
        self, channel_func, channel_func_unsub, channel_const, mock_connect
    ):
        # assert you can subscribe and unsubscribe to a channel
        mock_connect.return_value = self.mock_websocket

        # open
        await self.ws.open_async()
        self.assertIsNotNone(self.ws.websocket)

        # subscribe
        product_ids = []
        if channel_const not in NO_PRODUCT_CHANNELS:
            product_ids = ["BTC-USD", "ETH-USD"]
            await channel_func(product_ids=product_ids)
        else:
            await channel_func()
        self.mock_websocket.send.assert_awaited_once()

        # assert subscribe message
        subscribe = json.loads(self.mock_websocket.send.call_args_list[0][0][0])
        self.assertEqual(subscribe["type"], "subscribe")
        self.assertEqual(subscribe["product_ids"], product_ids)
        self.assertEqual(subscribe["channel"], channel_const)

        # unsubscribe
        if channel_const not in NO_PRODUCT_CHANNELS:
            await channel_func_unsub(product_ids=product_ids)
        else:
            await channel_func_unsub()
        self.assertEqual(self.mock_websocket.send.await_count, 2)

        # assert unsubscribe message
        unsubscribe = json.loads(self.mock_websocket.send.call_args_list[1][0][0])
        self.assertEqual(unsubscribe["type"], "unsubscribe")
        self.assertEqual(unsubscribe["product_ids"], product_ids)
        self.assertEqual(unsubscribe["channel"], channel_const)

        # close
        await self.ws.close_async()
        self.mock_websocket.close.assert_awaited_once()

    def test_heartbeats(self):
        self.generic_channel_test(
            self.ws.heartbeats, self.ws.heartbeats_unsubscribe, HEARTBEATS
        )

    def test_heartbeats_async(self):
        asyncio.run(
            self.generic_channel_test_async(
                self.ws.heartbeats_async,
                self.ws.heartbeats_unsubscribe_async,
                HEARTBEATS,
            )
        )

    def test_candles(self):
        self.generic_channel_test(self.ws.candles, self.ws.candles_unsubscribe, CANDLES)

    def test_candles_async(self):
        asyncio.run(
            self.generic_channel_test_async(
                self.ws.candles_async, self.ws.candles_unsubscribe_async, CANDLES
            )
        )

    def test_level2(self):
        self.generic_channel_test(self.ws.level2, self.ws.level2_unsubscribe, LEVEL2)

    def test_level2_async(self):
        asyncio.run(
            self.generic_channel_test_async(
                self.ws.level2_async, self.ws.level2_unsubscribe_async, LEVEL2
            )
        )

    def test_market_trades(self):
        self.generic_channel_test(
            self.ws.market_trades, self.ws.market_trades_unsubscribe, MARKET_TRADES
        )

    def test_market_trades_async(self):
        asyncio.run(
            self.generic_channel_test_async(
                self.ws.market_trades_async,
                self.ws.market_trades_unsubscribe_async,
                MARKET_TRADES,
            )
        )

    def test_status(self):
        self.generic_channel_test(self.ws.status, self.ws.status_unsubscribe, STATUS)

    def test_status_async(self):
        asyncio.run(
            self.generic_channel_test_async(
                self.ws.status_async, self.ws.status_unsubscribe_async, STATUS
            )
        )

    def test_ticker(self):
        self.generic_channel_test(self.ws.ticker, self.ws.ticker_unsubscribe, TICKER)

    def test_ticker_async(self):
        asyncio.run(
            self.generic_channel_test_async(
                self.ws.ticker_async, self.ws.ticker_unsubscribe_async, TICKER
            )
        )

    def test_ticker_batch(self):
        self.generic_channel_test(
            self.ws.ticker_batch, self.ws.ticker_batch_unsubscribe, TICKER_BATCH
        )

    def test_ticker_batch_async(self):
        asyncio.run(
            self.generic_channel_test_async(
                self.ws.ticker_batch_async,
                self.ws.ticker_batch_unsubscribe_async,
                TICKER_BATCH,
            )
        )

    def test_user(self):
        self.generic_channel_test(self.ws.user, self.ws.user_unsubscribe, USER)

    def test_user_async(self):
        asyncio.run(
            self.generic_channel_test_async(
                self.ws.user_async, self.ws.user_unsubscribe_async, USER
            )
        )

    def test_futures_balance_summary(self):
        self.generic_channel_test(
            self.ws.futures_balance_summary,
            self.ws.futures_balance_summary_unsubscribe,
            FUTURES_BALANCE_SUMMARY,
        )

    def test_futures_balance_summary_async(self):
        asyncio.run(
            self.generic_channel_test_async(
                self.ws.futures_balance_summary_async,
                self.ws.futures_balance_summary_unsubscribe_async,
                FUTURES_BALANCE_SUMMARY,
            )
        )



import json
import logging
import os
from typing import IO, Optional, Union

from coinbase.constants import API_ENV_KEY, API_SECRET_ENV_KEY


def get_logger(name):
    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)

    handler = logging.StreamHandler()
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s", "%Y-%m-%d %H:%M:%S"
    )
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    return logger


class APIBase(object):
    def __init__(
        self,
        api_key: Optional[str] = os.getenv(API_ENV_KEY),
        api_secret: Optional[str] = os.getenv(API_SECRET_ENV_KEY),
        key_file: Optional[Union[IO, str]] = None,
        base_url=None,
        timeout: Optional[int] = None,
        verbose: Optional[bool] = False,
    ):
        if (api_key is not None or api_secret is not None) and key_file is not None:
            raise Exception(f"Cannot specify both api_key and key_file in constructor")

        if key_file is not None:
            try:
                if isinstance(key_file, str):
                    with open(key_file, "r") as file:
                        key_json = json.load(file)
                else:
                    key_json = json.load(key_file)
                api_key = key_json["name"]
                api_secret = key_json["privateKey"]
            except json.JSONDecodeError as e:
                raise Exception(f"Error decoding JSON: {e}")

        self.is_authenticated = False
        if api_key is not None and api_secret is not None:
            self.api_key = api_key
            self.api_secret = bytes(api_secret, encoding="utf8").decode(
                "unicode_escape"
            )
            self.is_authenticated = True
        elif api_key is not None:
            raise Exception(
                f"Only api_key provided in constructor. Please also provide api_secret"
            )
        elif api_secret is not None:
            raise Exception(
                f"Only api_secret provided in constructor. Please also provide api_key"
            )

        self.base_url = base_url
        self.timeout = timeout



import unittest

from requests_mock import Mocker

from coinbase.rest import RESTClient

from ..constants import TEST_API_KEY, TEST_API_SECRET


class PaymentsTest(unittest.TestCase):
    def test_list_payment_methods(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"accounts": [{"uuid": "payment1"}, {"name": "payment2"}]}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/payment_methods",
                json=expected_response,
            )
            payments = client.list_payment_methods()

            captured_request = m.request_history[0]

            self.assertEqual(captured_request.query, "")
            self.assertEqual(payments, expected_response)

    def test_get_payment_method(self):
        client = RESTClient(TEST_API_KEY, TEST_API_SECRET)

        expected_response = {"uuid": "payment1"}

        with Mocker() as m:
            m.request(
                "GET",
                "https://api.coinbase.com/api/v3/brokerage/payment_methods/payment1",
                json=expected_response,
            )
            payment = client.get_payment_method("payment1")

            captured_request = m.request_history[0]

            self.assertEqual(captured_request.query, "")
            self.assertEqual(payment, expected_response)



from typing import Any, Dict

from coinbase.constants import API_PREFIX


def list_payment_methods(self, **kwargs) -> Dict[str, Any]:
    """
    **List Payment Methods**
    _________________
    [GET] https://api.coinbase.com/api/v3/brokerage/payment_methods

    __________

    **Description:**

    Get a list of payment methods for the current user.

    __________

    **Read more on the official documentation:** `List Payment Methods <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getpaymentmethods>`_
    """

    endpoint = f"{API_PREFIX}/payment_methods"

    return self.get(endpoint, **kwargs)


def get_payment_method(self, payment_method_id: str, **kwargs) -> Dict[str, Any]:
    """
    **Get Payment Method**
    _________________
    [GET] https://api.coinbase.com/api/v3/brokerage/payment_methods/{payment_method_id}

    __________

    **Description:**

    Get information about a payment method for the current user.

    __________

    **Read more on the official documentation:** `Get Payment Method <https://docs.cdp.coinbase.com/advanced-trade/reference/retailbrokerageapi_getpaymentmethod>`_
    """

    endpoint = f"{API_PREFIX}/payment_methods/{payment_method_id}"

    return self.get(endpoint, **kwargs)
